// Matrica.java - Klasa matrica numerikih podataka.

package aritmetika;

public class Matrica implements Numericko {
  private Numericko[][] matr;                     // Elementi matrice.
  private int m, n;                               // Dimenzije matrice.

  public Matrica(int mm, int nn)                  // Inicijalizacija.
    { matr = new Numericko [m=mm][n=nn]; }
                                                  // Postavljanje elementa.
  public Matrica postavi(int i, int j, Numericko a) throws GNumInd {
    if (i<0 || i>=m || j<0 || j>=n) throw new GNumInd();
    matr[i][j] = a; 
    return this; 
  }

  public Matrica clone() {                        // Kopija matrice.
    try {
      Matrica rez = (Matrica)super.clone();
      rez.matr = matr.clone();
      for (int i=0; i<m; i++) {
        rez.matr[i] = matr[i].clone();
        for (int j=0; j<n; j++)
          rez.matr[i][j] = matr[i][j].clone();
      }
      return rez;
    } catch (CloneNotSupportedException g)
      { return null; }
  }
                                                  // Dodavanje matrice.
  public Matrica dodaj(Numericko bb) throws GNumericka {
    if (!(bb instanceof Matrica)) throw new GNumTip();
    Matrica b = (Matrica)bb;
    if (m!=b.m || n!=b.n) throw new GNumDim();
    for (int i=0; i<m; i++)
      for (int j=0; j<n; j++)
        matr[i][j].dodaj(b.matr[i][j]);
    return this;
  }

  // Aktivna klasa za izraunavanje jednog elementa proizvoda matrica:
  private static class Mnozac extends Thread {
    Matrica a, b;                   // - matrice koje se mnoe,
    int i, k;                       // - vrsta prve i kolona druge matrice,
    Numericko cik;                  // - vrednost elementa rezultata,

    Mnozac(Matrica aa, Matrica bb, int ii, int kk)// - inicijalizacija,
      { a = aa; b = bb; i = ii; k = kk; start(); }

    public void run() {                           // - raunanje,
      try {
        cik = a.matr[i][0].proizvod(b.matr[0][k]);
        for (int j=1; j<a.n; j++)
          cik.dodaj(a.matr[i][j].proizvod(b.matr[j][k]));
      } catch (GNumericka g) { cik = null; }
    }

    Numericko cik() { return cik; }               // - dohvatanje rezultata.
  } // class Mnozac
                                                  // Proizvod dve matrice.
  public Matrica proizvod(Numericko bb) throws GNumericka {
    if (!(bb instanceof Matrica)) throw new GNumTip();
    Matrica b = (Matrica)bb;
    if (n != b.m) throw new GNumDim();
    Matrica c = new Matrica(m, b.n);
    Mnozac[][] mno = new Mnozac[c.m][c.n];
    for (int i=0; i<c.m; i++)
      for (int k=0; k<c.n; k++)
        mno[i][k] = new Mnozac(this, b, i, k);
    try {
      for (int i=0; i<c.m; i++)
        for (int k=0; k<c.n; k++) {
          mno[i][k].join();
          c.matr[i][k] = mno[i][k].cik();
        }
    } catch (InterruptedException g) { c = null; }
    return c;
  }

  public String toString() {                      // Tekstualni opis.
    StringBuilder s = new StringBuilder();
    for (int i=0; i<m; i++)
      for (int j=0; j<n; j++) {
        s.append(matr[i][j]);
        s.append(j<n-1 ? ' ' : '\n');
      }
    return s.toString();
  }
}
